# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['metadamage', 'metadamage.dashboard']

package_data = \
{'': ['*']}

install_requires = \
['PyPDF2>=1.26.0,<2.0.0',
 'about-time>=3.1.1,<4.0.0',
 'click-help-colors>=0.9,<0.10',
 'dash-bootstrap-components>=0.11.3,<0.12.0',
 'dash-extensions>=0.0.53,<0.0.54',
 'dash>=1.19.0,<2.0.0',
 'dask[complete]>=2021.3.0,<2022.0.0',
 'dill>=0.3.3,<0.4.0',
 'ete3>=3.1.2,<4.0.0',
 'fsspec>=0.8.5,<0.9.0',
 'iminuit>=2.4.0,<3.0.0',
 'ipywidgets>=7.5',
 'joblib>=1.0.0,<2.0.0',
 'jupyterlab>=3.0.7,<4.0.0',
 'lazy-object-proxy==1.4.3',
 'matplotlib>=3.3.3,<4.0.0',
 'memory-profiler>=0.58.0,<0.59.0',
 'notebook>=5.3',
 'numpyro>=0.4.1,<0.5.0',
 'pandas>=1.2.0,<2.0.0',
 'partd>=1.1.0,<2.0.0',
 'pip>=20.3.3,<21.0.0',
 'plotly>=4.14.3,<5.0.0',
 'poetry-dynamic-versioning>=0.12.1,<0.13.0',
 'pyarrow>=2.0.0,<3.0.0',
 'rich>=9.8.1,<10.0.0',
 'tables>=3.6.1,<4.0.0',
 'timeout-decorator>=0.5.0,<0.6.0',
 'toml>=0.10.2,<0.11.0',
 'tornado>=6.1,<7.0',
 'typer[all]>=0.3.2,<0.4.0']

extras_require = \
{':python_version >= "3.8" and python_version < "3.10"': ['numba>=0.53.1,<0.54.0']}

entry_points = \
{'console_scripts': ['metadamage = metadamage.cli:cli_main']}

setup_kwargs = {
    'name': 'metadamage',
    'version': '0.5.2',
    'description': 'Metagenomics Ancient Damage: metadamage',
    'long_description': '# Metadamage - Ancient Damage Estimation for Metagenomics\n\n[![PyPI](https://img.shields.io/pypi/v/metadamage)](https://pypi.org/project/metadamage) ![PyPI - License](https://img.shields.io/pypi/l/metadamage)\n\n---\n\n#### Work in progress. Please contact christianmichelsen@gmail.com for further information.\n\n---\n\n\nPersonal recommendations for this project:\n\n- Python Version: [Pyenv](https://github.com/pyenv/pyenv)\n- Virtual Environment:  [Virtualenv](https://github.com/pyenv/pyenv-virtualenv)\n- Package Management: [Poetry](https://python-poetry.org/)\n\nThis project requires a recent Python version (>=3.8) installed. See more [below](#setup).\n\n\n## <a name="install"></a> Installation:\n\nWith Poetry:\n```console\n$ poetry add metadamage\n```\nor, if you prefer regular pip:\n```console\n$ pip install metadamage\n```\n\n## <a name="cli"></a> Metadamage CLI:\n\nFor help on the CLI interface, `metadamage` provides a `help` function:\n\n```console\n$ metadamage --help\n```\n\nExample for fitting a single file using `metadamage`:\n```console\n$ metadamage fit --max-fits 10 --max-cores 2 ./data/input/data_ancient.txt\n```\n\n`metadamage` also allows to fit multiple files:\n```console\n$ metadamage fit --max-fits 10 --max-cores 2 ./data/input/*.txt\n```\n\n## <a name="dash"></a> Dashboard:\n\nTo make use of the new, interactive dashboard introduced in version `0.4`, run the following command (after having [fitted](#cli) the files):\n\n```console\n$ metadamage dashboard\n```\n\nAnd then open a browser and go to `127.0.0.1:8050` (if it did not open automatically). For more information, use:\n```console\n$ metadamage dashboard --help\n```\n\n## <a name="dash-server"></a> Dashboard on a server:\n\nIf you are running metadamage on a server and want to use the dashboard locally, you can setup a SSH tunnel. First, on the server, run `metadamage dashboard` with the relevant options (and keep it open, e.g. with TMUX). Afterwards, on your local machine, run:\n\n```console\n$ ssh -L 8050:127.0.0.1:8050 -N user@remote\n```\nNow you can open a browser and go to [`http://0.0.0.0:8050`](http://0.0.0.0:8050).\n\nIn case you\'re connecting through a jump host, you can use the the `-J` option:\n```console\n$ ssh -L 8050:127.0.0.1:8050 -N -J user@jumphost user@remote\n```\n\nFor an easier method, you can setup your ssh config (usually at `~/.ssh/config`) in the following way:\n\n```ssh-config\nHost jumphost\n    User your-jumphost-username-here\n    HostName your-jumphost-address-here\n\nHost remote\n\nHost dashboard\n    Port 22\n    LocalForward 8050 localhost:8050\n    RemoteCommand echo "Connecting to dashboard ... CTRL+C to terminate"; sleep infinity\n\nHost remote dashboard\n    ProxyJump jumphost\n    User your-remote-username-here\n    HostName your-remote-address-here\n```\n\nNow if you simply run the following on your own computer (in a new terminal session):\n\n```console\n$ ssh dashboard\n```\nyou can open open a browser and go to [`http://0.0.0.0:8050`](http://0.0.0.0:8050).\n\n<!-- ssh -L 8050:127.0.0.1:8050 -N -J willerslev mnv794@wonton-snm -->\n<!-- ssh -L 8050:127.0.0.1:8050 -N -J mnv794@ssh-snm-willerslev.science.ku.dk mnv794@wonton-snm -->\n\n<!-- ssh -L 8050:127.0.0.1:8050 -N hep -->\n\n\n\n## <a name="options"></a> Metadamage CLI fit options:\n\nThe `metadamage fit` CLI has the following options.\n\n- Output directory\n  - `--out_dir`: The directory in which the fit results are stored. Default location is `./data/out`. Do not change unless you known what you are doing.\n\n- Maximum values\n  - `--max-fits`: Maximum number of fits to do. Default is None, i.e. fit everything.\n  - `--max-cores`: Maximum number of cores to use while fitting. Default is 1.\n  - `--max-position`: Maximum position in the sequence to include. Default is +/- 15 (forward/reverse).\n\n- Minimum values or cuts/thresholds for plots\n  - `--min-alignments`: Minimum number of alignments (![N_\\mathrm{alignments}](https://render.githubusercontent.com/render/math?math=%5Clarge+%5Cdisplaystyle+N_%5Cmathrm%7Balignments%7D)\n) of a single TaxID to be fitted. Default is 10.\n  - `--min-k-sum`: Minimum sum of `k` of a single TaxID to be fitted. Here `k` might be e.g. the number of A→T transitions in the forward direction and the number of G→A transitions in the reverse direction.  In that case, it would be: ![\\mathtt{k} = \\sum_{z=1}^{15} \\left( N_{\\mathrm{CT}}(z)  +  N_{\\mathrm{GA}}(-z) \\right)](https://render.githubusercontent.com/render/math?math=%5Ctextstyle+%5Cmathtt%7By%7D+%3D+%5Csum_%7Bz%3D1%7D%5E%7B15%7D+%5Cleft%28+N_%7B%5Cmathrm%7BCT%7D%7D%28z%29++%2B++N_%7B%5Cmathrm%7BGA%7D%7D%28-z%29+%5Cright%29).  Default is 10.\n- `--min-N-at-each-pos`: Minimum number of N at each position.\n\n- Other:\n  - `--substitution-bases-forward`:  Which substitution to check for damage in the forward region. Do not change this value except for control checks. Default is `CT`.\n  - `--substitution-bases-reverse`:  Which substitution to check for damage in the reverse region. Do not change this value except for control checks. Default is `GA`.\n\n- Boolean Flags\n  - `--bayesian`: Include a fully Bayesian model (probably better, but also _a lot_ slower, about a factor of 100).\n  - `--forced`: Force redo everything (count data and fits).\n  - `--dask-port`: Local port used for Dask. Integer. Default is 8787.\n  <!-- - `--force-reload-files`: Force reload the original datafile (and thus not load the autogenerated parquet file). -->\n  <!-- - `--force-plots`: Force recreate  the plots, even though the plots already exist. -->\n  <!-- - `--force-no-plots`: Force not to make any plots at all, even though `max-plots` exists (and is >0). -->\n\n  - `--version`: Print the current version of the program and exit.\n\n\n<!-- [tex-image-link-generator](https://tex-image-link-generator.herokuapp.com/) -->\n<!-- https://gist.github.com/a-rodin/fef3f543412d6e1ec5b6cf55bf197d7b -->\n\n---\n\n## <a name="setup"></a> Setup Local Python Environment:\n\nMake sure you have a local Python environment. Personally, I recommend using Pyenv for installing Python versions and Pyenv-Virtualenv for easy managing of virtuel environments. See e.g. [this](https://github.com/pyenv/pyenv-installer#pyenv-installer) for easy installation of both.\n\nMake sure you have a decent Python version (>=3.8) installed:\n```console\n$ pyenv install 3.8.7\n```\n\nNow we set up a virtual environment, such that changes you do in this environment does not affect your other Python projects:\n```console\n$ pyenv virtualenv 3.8.7 metadamage38\n$ pyenv activate metadamage38\n```\n\nWe now use Poetry to setup a new project which uses metadamage. Follow the interactive guide:\n```console\n$ poetry new metadamage-folder\n$ cd metadamage-folder\n```\n\nInstead of activating the environment manually after every new login, we can tell `pyenv` to remember it for us:\n```console\n$ pyenv local metadamage38\n```\n\nWe now have a working local, virtual Python environment where the packages are managed by Poetry, so we can now add metadamamage to our project:\n```console\n$ poetry add metadamage\n```\n\nAt this point you should log out of your terminal and log in again for reloading everything. Now if you just write:\n\n```console\n$ metadamage\n```\n\nyou should see the following:\n\n```console\n$ metadamage\nUsage: metadamage [OPTIONS] FILENAMES...\nTry \'metadamage --help\' for help.\n\nError: Missing argument \'FILENAMES...\'.\n```\n\nwhich shows that it is working and installed. You can now use `metadamage --help` for more help (together with the variable explanations [above](#options)).\n\n---\n\n## <a name="update"></a> Update:\n\nWith Poetry:\n```console\n$ poetry update metadamage\n```\nor, if you prefer regular pip:\n```console\n$ pip install metadamage --upgrade\n```\n\n---\n\n## <a name="dev_branch"></a> Development Branch:\n\nYou can also use a newer version directly from Github:\n```console\n$ poetry add git+https://github.com/ChristianMichelsen/metadamage.git\n```\nor a specific branch (named `BRANCH`):\n```console\n$ poetry add git+https://github.com/ChristianMichelsen/metadamage.git#BRANCH\n```\n\n---\n\n## Conda:\n\nIf you prefer using Conda, you can also install `metadamage` (via pip). First create a folder:\n```console\n$ mkdir metadamage-conda\n$ cd metadamage-conda\n```\n\nTo install `metadamage`:\n```console\n$ wget https://raw.githubusercontent.com/ChristianMichelsen/metadamage/main/environment.yaml\n$ conda env create -f environment.yaml\n```\n\nTo update it to a new, released version of `metadamage`:\n```console\n$ wget https://raw.githubusercontent.com/ChristianMichelsen/metadamage/main/environment.yaml\n$ conda env update --file environment.yaml\n```\n\nFinally remember to activate the environment:\n```console\n$ conda activate metadamage\n```\n\n<!-- poetry add git+https://github.com/christianmichelsen/metadamage.git -->\n<!-- poetry add git+https://github.com/christianmichelsen/metadamage.git#develop -->\n',
    'author': 'Christian Michelsen',
    'author_email': 'christianmichelsen@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ChristianMichelsen/metadamage',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
